@@ -14,8 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * License along with this library. If not, see
+ * <http://www.gnu.org/licenses/>.
  *
  * Authors:
  *   Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
@@ -27,6 +27,8 @@
 #ifndef __OSINFO_INSTALL_SCRIPT_PRIVATE_H__
 #define __OSINFO_INSTALL_SCRIPT_PRIVATE_H__
 
+void osinfo_install_script_add_config_param(OsinfoInstallScript *script, OsinfoInstallConfigParam *param);
+
 void osinfo_install_script_set_avatar_format(OsinfoInstallScript *script,
                                              OsinfoAvatarFormat *avatar);
 
