@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2012 Red Hat, Inc.
+ * Copyright (C) 2009-2012, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,6 +93,7 @@ static struct ISOInfo *load_iso(GFile *file, const gchar *shortid, const gchar *
     GDataInputStream *dis = NULL;
     gchar *line;
     const gchar *arch;
+    gint64 vol_size = -1, blk_size;
 
     if (!(fis = g_file_read(file, NULL, error)))
         goto error;
@@ -148,9 +149,18 @@ static struct ISOInfo *load_iso(GFile *file, const gchar *shortid, const gchar *
             osinfo_entity_set_param(OSINFO_ENTITY(info->media),
                                     OSINFO_MEDIA_PROP_APPLICATION_ID,
                                     line + 16);
+        } else if (g_str_has_prefix(line, "Logical block size is:")) {
+            blk_size = (gint64) atoll(line + 23);
+        } else if (g_str_has_prefix(line, "Volume size is:")) {
+            vol_size = atoll(line + 16);
         }
     }
 
+    if (vol_size > 0)
+        osinfo_entity_set_param_int64(OSINFO_ENTITY(info->media),
+                                      OSINFO_MEDIA_PROP_VOLUME_SIZE,
+                                      vol_size * blk_size);
+
     if (*error)
         goto error;
 
@@ -408,6 +418,18 @@ START_TEST(test_mageia)
 }
 END_TEST
 
+START_TEST(test_sles)
+{
+    test_one("sles");
+}
+END_TEST
+
+START_TEST(test_sled)
+{
+    test_one("sled");
+}
+END_TEST
+
 static Suite *
 list_suite(void)
 {
@@ -426,6 +448,8 @@ list_suite(void)
     tcase_add_test(tc, test_gnome);
     tcase_add_test(tc, test_altlinux);
     tcase_add_test(tc, test_mageia);
+    tcase_add_test(tc, test_sles);
+    tcase_add_test(tc, test_sled);
     suite_add_tcase(s, tc);
     return s;
 }
@@ -433,8 +457,8 @@ list_suite(void)
 int main(void)
 {
     int number_failed;
-    Suite *s = list_suite ();
-    SRunner *sr = srunner_create (s);
+    Suite *s = list_suite();
+    SRunner *sr = srunner_create(s);
 
 #if !GLIB_CHECK_VERSION(2,35,1)
     g_type_init();
@@ -452,9 +476,9 @@ int main(void)
     osinfo_oslist_get_type();
     osinfo_filter_get_type();
 
-    srunner_run_all (sr, CK_ENV);
-    number_failed = srunner_ntests_failed (sr);
-    srunner_free (sr);
+    srunner_run_all(sr, CK_ENV);
+    number_failed = srunner_ntests_failed(sr);
+    srunner_free(sr);
 
     return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
